#include "instance.h"
#include <algorithm>
#include <memory>
#include <optional>
#include <vector>

std::optional<std::shared_ptr<Instance>> Instance::GetParent() {
    if (!this->Parent.has_value() || this->Parent.value().expired())
        return std::nullopt;
    return this->Parent.value().lock();
}

void Instance::SetParent(std::optional<std::shared_ptr<Instance>> newParent) {
    // Remove this instance from the existing parent
    if (this->Parent.has_value() && !this->Parent.value().expired()) {
        std::shared_ptr<Instance> parent = this->Parent.value().lock();
        parent->children.erase(std::remove_if(parent->children.begin(), parent->children.end(), [this](std::shared_ptr<Instance> ptr) {
            return ptr.get() == this;
        }));
    }

    // Add self to 
    if (newParent.has_value()) {
        newParent.value()->children.push_back(this->shared_from_this());
    }

    this->Parent = newParent;
}

std::vector<std::shared_ptr<Instance>> Instance::GetChildren() {
    return this->children;
}

std::unique_ptr<Instance> Instance::CloneInternal() {
    // TODO: Implement refs

    std::unique_ptr<Instance> clone = this->copy();
    clone->children = std::vector<std::shared_ptr<Instance>>();
    clone->children.reserve(this->children.size());

    for (std::shared_ptr<Instance> child : this->children) {
        clone->children.push_back(std::make_shared<Instance>(child->CloneInternal()));
    }

    return clone;
}